/*
    Copyright 2004,2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>



#define VER             "0.1"
#define BUFFSZ          4096
#define PASS            "SavePasswordText="
#define EUDORADATA  \
    "\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFF\xFF\xFE\xFE\xFF\xFE\xFE" \
    "\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE" \
    "\xFF\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\x3E\xFE\xFE\xFE\x3F" \
    "\x34\x35\x36\x37\x38\x39\x3A\x3B\x3C\x3D\xFE\xFE\xFE\xFD\xFE\xFE" \
    "\xFE\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0A\x0B\x0C\x0D\x0E" \
    "\x0F\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\xFE\xFE\xFE\xFE\xFE" \
    "\xFE\x1A\x1B\x1C\x1D\x1E\x1F\x20\x21\x22\x23\x24\x25\x26\x27\x28" \
    "\x29\x2A\x2B\x2C\x2D\x2E\x2F\x30\x31\x32\x33\xFE\xFE\xFE\xFE\xFE" \
    "\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE" \
    "\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE" \
    "\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE" \
    "\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE" \
    "\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE" \
    "\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE" \
    "\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE" \
    "\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE"



#ifndef WIN32
    #define strnicmp strncasecmp
#endif

#ifndef u_char
    #define u_char  unsigned char
#endif



void eudora_pwddec(u_char *pwd);
void std_err(void);



int main(int argc, char *argv[]) {
    FILE    *fd;
    u_char  buff[BUFFSZ + 1],
            *ptr,
            *ptr1;


    setbuf(stdout, NULL);

    fputs("\n"
        "Eudora (tested 6.0) passwords decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\nUsage: %s <Eudora.ini>\n"
            "\n", argv[0]);
        fputs("\n    Press RETURN to exit\n", stdout);
        fgetc(stdin);
        exit(1);
    }

    printf("\nOpening file %s\n\n", argv[1]);
    fd = fopen(argv[1], "rb");
    if(!fd) std_err();

    while(1) {
        if(!fgets(buff, BUFFSZ, fd)) break;

        if(strnicmp(buff, PASS, sizeof(PASS) - 1)) continue;

        ptr = ptr1 = buff + sizeof(PASS) - 1;
        for(; *ptr > 0xd; ptr++);   // \r\n
        *ptr = 0;

        eudora_pwddec(ptr1);
        printf("Password:   %s\n", ptr1);
    }

    fclose(fd);
    fputs("\n\n    Press RETURN to exit\n", stdout);
    fgetc(stdin);
    return(0);
}



void eudora_pwddec(u_char *pwd) {
    int     len;
    u_char  *out,
            eax,
            esic = 0,
            ebpc = 0;
    int     ecx,
            esi4 = 0,
            esi8 = 0,
            ebpm4 = 0;

    len = strlen(pwd);
    out = pwd;
    while(len--) {
        eax = EUDORADATA[*pwd++];
        ecx = eax - 0xfd;
        ebpc = eax;

        if(!ecx) {
            esi8++;
            continue;
        } else if(!--ecx) {
            ebpm4++;
            continue;
        } else if(!--ecx) {
            continue;
        }

        if(esi8) {
            ebpm4 += esi8;
            esi8 = 0;
        }

        ecx = esi4;
        if(!ecx) {
            esic = eax << 2;
            esi4 = 1;
            continue;
        } else if(!--ecx) {
            eax = (ebpc >> 4) | esic;
            *out++ = eax;
			esic = (ebpc & 0xf) << 4;
            esi4++;
            continue;
        } else if(!--ecx) {
            eax = (ebpc >> 2) | esic;
            *out++ = eax;
            esic = (ebpc & 3) << 6;
            esi4++;
            continue;
        } else if(--ecx) {
            continue;
        }
        *out++ = esic | eax;
        esi4 = 0;
    }
    *out = 0;
}



void std_err(void) {
    perror("\nError");
    fputs("\n    Press RETURN to exit\n", stdout);
    fgetc(stdin);
    exit(1);
}


